-- by mor2000

--------------------
-- filltypechanger

UPK_Filltypechanger={}
local UPK_Filltypechanger_mt = ClassUPK(UPK_Filltypechanger,UPK_DisplayTrigger)
InitObjectClass(UPK_Filltypechanger, "UPK_Filltypechanger")
UniversalProcessKit.addModule("filltypechanger",UPK_Filltypechanger)

function UPK_Filltypechanger:new(isServer, isClient, customMt)
	local self = UPK_Filltypechanger:superClass().new(self, isServer, isClient, customMt or UPK_Filltypechanger_mt)
	registerObjectClassName(self, "UPK_Filltypechanger")
	return self
end

function UPK_Filltypechanger:load(id)
	if not UPK_Filltypechanger:superClass().load(self, id, nil) then
		self:print('Error: loading filltypechanger failed',true)
		return false
	end
	
	for k,_ in pairs(self.acceptedFillTypes) do
		if k>=1025 then
			rawset(self.acceptedFillTypes,k,nil)
		end
	end
	
	self.tiptriggerFillTypes={} -- filltypes known to tip- and dumptrigger
	
	self.fillTypesToSilos={}
	
	self.fillLevels={}
	setmetatable(self.fillLevels, {
		__index = function(t,k)
			return (self.fillTypesToSilos[k] or {}).fillLevel or 0
		end
	})

	local function tiptriggersetFillLevel(tiptrigger, fillLevel, fillType)
		if self.fillTypesToSilos[fillType]~=nil then
			return self.fillTypesToSilos[fillType]:setFillLevel(fillLevel, fillType)
		end
		return 0
	end;

	local function tiptriggeraddFillLevel(tiptrigger, deltaFillLevel, fillType)
		if self.fillTypesToSilos[fillType]~=nil then
			return self.fillTypesToSilos[fillType]:addFillLevel(deltaFillLevel, fillType)
		end
		return 0
	end;
	
	self.storageKids={}
	self.nrStorageKids=0
	for _,v in pairs(self.kids) do
		if v.type=="storage" then
			self.nrStorageKids=self.nrStorageKids+1
			table.insert(self.storageKids,v)
		elseif v.type=="tiptrigger" then
			v.acceptedFillTypes=self.tiptriggerFillTypes -- redirect table lookups
			v.setFillLevel=tiptriggersetFillLevel
			v.addFillLevel=tiptriggeraddFillLevel
		elseif v.type=="dumptrigger" then
			v.acceptedFillTypes=self.tiptriggerFillTypes -- redirect table lookups
			v.setFillLevel=tiptriggeraddFillLevel
			v.addFillLevel=tiptriggeraddFillLevel
		end
	end
	
	
	if self.nrStorageKids==0 then
		self:print('Error: filltypechanger needs at least one storage module as child')
		return false
	end
	
	self.silosInUse={}
	
	self.activeSilo=0
	self.nextActiveSilo=nil
	
	self.watchtime=0
	self.canPressNextKey=false
			
	--self:getAcceptedFillTypes()

	self:print('loaded filltypechanger successfully')
	return true
end

function UPK_Filltypechanger:delete()
	UPK_Filltypechanger:superClass().delete(self)
end

function UPK_Filltypechanger:update(dt)
	UniversalProcessKit.update(self,dt)
	
	self.watchtime=self.watchtime+dt
	local nextFruitType=nil
		
	if self.isClient then
		if self:getShowInfo() then
			-- find silos that are empty
			local emptySilos={}
			local fillTypesInUse={}
			for k,v in pairs(self.storageKids) do
				table.insert(fillTypesInUse,v.fillType,true)
				if v.fillLevel>0 then
					table.insert(self.silosInUse,k,true)
					if self.activeSilo==k then
						self.activeSilo=0
					end
				else
					self.silosInUse[k]=nil
					table.insert(emptySilos,k)
				end	
			end
			
			
	
			-- set active silo if none or only one silo is chooseable
			if(#emptySilos==0)then
				self.activeSilo=0
				self.nextActiveSilo=nil
			elseif(#emptySilos==1)then
				self.activeSilo=emptySilos[1]
				self.nextActiveSilo=nil
			else
				-- determine next silo to choose
				self.nextActiveSilo=nil
				for k,v in pairs(emptySilos)do
					if(v>self.activeSilo)then
						self.nextActiveSilo=v
						break
					end
				end
				if(self.nextActiveSilo==nil and emptySilos[1]~=self.activeSilo)then
					self.nextActiveSilo=emptySilos[1]
				end
			end
			
			
			local minFillType=0
			local nextFillType=0
			if self.activeSilo~=0 then
				local activeFillType=self.storageKids[self.activeSilo].fillType
				local acceptedFillTypes=self:getAcceptedFillTypes()
				table.sort(acceptedFillTypes)
				for _,v in pairs(acceptedFillTypes) do
					if not fillTypesInUse[v] then
						if v>0 and minFillType==0 then
							minFillType=v
						end
						if v>activeFillType then
							nextFillType=v
							break
						end
					end
				end
				if nextFillType==0 then
					nextFillType=minFillType
				end
			end
			
			for k,v in pairs(self.storageKids) do
				local fillType=v.fillType
				local fillLevel=v.fillLevel
				local fluid_unit_short=g_i18n:getText("fluid_unit_short")

				local text=g_i18n:getText("silo").." "..tostring(k)..": "

				if fillLevel>0 then
					text=text..tostring(math.max(0,math.floor(fillLevel+0.5))) .. "[" .. fluid_unit_short .. "] "
					local capacity=v.capacity
					if capacity<math.huge then
						text=text.." "..math.max(0,math.ceil(fillLevel/capacity*100)) .. "% "
					end
				end
				
				local i18n_key=UniversalProcessKit.fillTypeIntToName[fillType]
				if g_i18n:hasText(i18n_key) then
					text=text..g_i18n:getText(i18n_key)
				else
					text=text..i18n_key
				end

				if self.activeSilo==k and nextFillType~=0 and fillLevel==0 then
					local i18n_key=UniversalProcessKit.fillTypeIntToName[nextFillType]
					if g_i18n:hasText(i18n_key) then
						text=text.." -> "..g_i18n:getText(i18n_key)
					else
						text=text.." -> "..i18n_key
					end
				end
				
    			g_currentMission:addExtraPrintText(text)
			end
			
			
			if self.nextActiveSilo~=nil then
				g_currentMission:addHelpButtonText(g_i18n:getText("nextSilo"), InputBinding.PLACEABLESILO_NEXTSILO)
				if self.canPressNextKey and InputBinding.isPressed(InputBinding.PLACEABLESILO_NEXTSILO) then
					self.activeSilo=self.nextActiveSilo
					self.watchtime=0
					self.canPressNextKey=false
				end
			end
			
			if self.activeSilo~=nil then
				if nextFillType~=0 then
					g_currentMission:addHelpButtonText(g_i18n:getText("changeFruitType"), InputBinding.PLACEABLESILO_CHANGEFRUITTYPE)
					if self.canPressNextKey and InputBinding.isPressed(InputBinding.PLACEABLESILO_CHANGEFRUITTYPE) then
						self.storageKids[self.activeSilo]:setFillType(nextFillType)
						self:setAcceptedFillTypes()
						self.watchtime=0
						self.canPressNextKey=false
					end
				end
			end
			
		end
	end
	
	if(self.watchtime>300)then
		self.watchtime=0
		self.canPressNextKey=true
	end
end;

function UPK_Filltypechanger:setAcceptedFillTypes()
	self:print('UPK_Filltypechanger:setAcceptedFillTypes')
	self.fillTypesToSilos={}
	for k,_ in pairs(self.acceptedFillTypes) do
		self.tiptriggerFillTypes[k]=false
	end
	for k,v in pairs(self.storageKids) do
		self.tiptriggerFillTypes[v.fillType]=true
		self.fillTypesToSilos[v.fillType]=v
	end
end;

function UPK_Filltypechanger:loadFromAttributesAndNodes(xmlFile, key)
	if UPK_Filltypechanger:superClass().loadFromAttributesAndNodes(self, xmlFile, key) then
		self:setAcceptedFillTypes()
		return true
	end
	return false
end;
